/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2007 Pierre Habouzit
 */

#include <lib-lib/lib-lib.h>
#include "lib-lua_priv.h"

static lua_State *L;

static const luaL_Reg lualibs[] = {
    {"",              luaopen_base},
    {LUA_OSLIBNAME,   luaopen_os},
    {LUA_LOADLIBNAME, luaopen_package},
    {LUA_TABLIBNAME,  luaopen_table},
    {LUA_IOLIBNAME,   luaopen_io},
    {LUA_STRLIBNAME,  luaopen_string},
    {LUA_MATHLIBNAME, luaopen_math},
    {LUA_DBLIBNAME,   luaopen_debug},
    {LUA_MADMUTT,     luaopen_madmutt},
};

void mlua_initialize(void)
{
    int i;

    L = lua_open();
    for (i = 0; i < countof(lualibs); i++) {
        lua_pushcfunction(L, lualibs[i].func);
        lua_pushstring(L, lualibs[i].name);
        lua_call(L, 1, 0);
    }
}

void mlua_shutdown(void)
{
    lua_close(L);
}


int mlua_dofile(const char *filename)
{
    return luaL_dofile(L, filename);
}

int mlua_wrap(void (*errfun)(const char *fmt, ...), int status)
{
    if (status) {
        (*errfun)("-[lua]-: %s\n", lua_tostring(L, -1));
        lua_pop(L, 1);
    }
    return status;
}

ssize_t mlua_value(char *buf, ssize_t len,
                   const char *table, const char *key)
{
    ssize_t res;

    lua_getglobal(L, table);
    lua_pushstring(L, key);
    lua_gettable(L, -2);
    res = m_strcpy(buf, len, lua_tostring(L, -1));
    lua_remove(L, -1);
    lua_remove(L, -1);

    return res;
}

